from typing import List, Literal, Optional, Union

import msgspec

from betfair_parser.spec.api.core import APIBase, RequestBase


# ------------ ORDER TYPES ------------ #


class LimitOrder(msgspec.Struct):
    size: Union[str, float]
    price: Union[str, float]
    persistenceType: Literal["PERSIST"]


class MarketOnCloseOrder(msgspec.Struct):
    liability: str


# ------------ REQUESTS ------------ #


class placeInstructions(msgspec.Struct):
    selectionId: str
    handicap: str
    customerOrderRef: str
    orderType: Literal["LIMIT", "MARKET_ON_CLOSE"]
    side: Literal["BACK", "LAY"]
    limitOrder: Optional[LimitOrder] = None
    marketOnCloseOrder: Optional[MarketOnCloseOrder] = None


class placeOrdersParams(msgspec.Struct):
    marketId: str
    instructions: List[placeInstructions]
    customerRef: Optional[str]
    customerStrategyRef: Optional[str]


class placeOrders(RequestBase, kw_only=True):
    params: placeOrdersParams
    method: Literal["SportsAPING/v1.0/placeOrders"] = "SportsAPING/v1.0/placeOrders"


class cancelOrdersInstructions(msgspec.Struct):
    betId: str


class cancelOrdersParams(msgspec.Struct):
    marketId: str
    instructions: List[cancelOrdersInstructions]
    customerRef: str


class cancelOrders(RequestBase, kw_only=True):
    method: Literal["SportsAPING/v1.0/cancelOrders"] = "SportsAPING/v1.0/cancelOrders"
    params: cancelOrdersParams


class replaceOrdersInstructions(msgspec.Struct):
    betId: str
    newPrice: float


class replaceOrdersParams(msgspec.Struct):
    marketId: str
    instructions: List[replaceOrdersInstructions]
    customerRef: str


class replaceOrders(RequestBase, kw_only=True):
    method: Literal["SportsAPING/v1.0/replaceOrders"] = "SportsAPING/v1.0/replaceOrders"
    params: replaceOrdersParams


# ------------ RESPONSES ------------ #

STATUS = Literal["SUCCESS", "FAILURE"]
ORDER_STATUS = Literal["EXECUTABLE", "EXECUTION_COMPLETE"]


class Instruction(APIBase):
    selectionId: int
    handicap: float
    limitOrder: LimitOrder


class InstructionReport(APIBase):
    status: STATUS
    instruction: Instruction
    betId: Optional[str] = None
    placedDate: Optional[str] = None
    averagePriceMatched: Optional[float] = None
    sizeMatched: Optional[float] = None
    orderStatus: Optional[ORDER_STATUS] = None


class PlaceResult(APIBase):
    customerRef: str
    status: STATUS
    marketId: str
    instructionReports: Optional[List[InstructionReport]]


class PlaceResultResponse(RequestBase, kw_only=True):
    result: PlaceResult


class ReplaceResultResponse(RequestBase, kw_only=True):
    result: PlaceResult


__all__ = ["placeOrders", "cancelOrders", "replaceOrders"]



EVENT_TYPE_TO_NAME = {
    "1": "Soccer",
    "2": "Tennis",
    "3": "Golf",
    "4": "Cricket",
    "5": "Rugby Union",
    "1477": "Rugby League",
    "6": "Boxing",
    "7": "Horse Racing",
    "8": "Motor Sport",
    "27454571": "Esports",
    "10": "Special Bets",
    "998917": "Volleyball",
    "11": "Cycling",
    "2152880": "Gaelic Games",
    "3988": "Athletics",
    "6422": "Snooker",
    "7511": "Baseball",
    "6231": "Financial Bets",
    "6423": "American Football",
    "7522": "Basketball",
    "7524": "Ice Hockey",
    "61420": "Australian Rules",
    "468328": "Handball",
    "3503": "Darts",
    "26420387": "Mixed Martial Arts",
    "4339": "Greyhound Racing",
    "2378961": "Politics",
}



from typing import Union

from msgspec.json import Decoder

from betfair_parser.spec.streaming.mcm import MCM
from betfair_parser.spec.streaming.ocm import OCM
from betfair_parser.spec.streaming.status import Connection, Status


STREAM_MESSAGE = Union[Connection, Status, MCM, OCM]
STREAM_DECODER = Decoder(STREAM_MESSAGE)



from typing import Literal, Optional

import msgspec


class Connection(msgspec.Struct, tag_field="op", tag=str.lower):
    """
    Connection Message
    """

    connectionId: str


class Status(msgspec.Struct, tag_field="op", tag=str.lower):
    """
    Status Message
    """

    statusCode: Literal["SUCCESS", "FAILURE"]
    connectionClosed: bool
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    connectionsAvailable: Optional[int] = None



import fsspec
import msgspec

from betfair_parser.spec.streaming import STREAM_DECODER, STREAM_MESSAGE


def parse(line: bytes) -> STREAM_MESSAGE:
    return STREAM_DECODER.decode(line)


def read_file(fsspec_url: str):
    with fsspec.open(fsspec_url, compression="infer") as f:
        for line in f:
            try:
                data = parse(line)
            except (msgspec.DecodeError, msgspec.ValidationError) as e:
                print("ERR", e)
                print(msgspec.json.decode(line))
                raise e
            yield data



from typing import Literal, Optional, Union

import msgspec


class NavigationMarket(msgspec.Struct, kw_only=True):  # type: ignore
    """NavigationMarket"""

    event_type_name: str
    event_type_id: str
    event_name: Optional[str] = None
    event_id: Optional[str] = None
    event_countryCode: Optional[str] = None
    market_name: str
    market_id: str
    market_exchangeId: str
    market_marketType: str
    market_marketStartTime: str
    market_numberOfWinners: Union[str, int]


class PriceLadderDescription(msgspec.Struct):
    """PriceLadderDescription"""

    type: Literal["CLASSIC", "LINE_RANGE", "FINEST"]


class LineRangeInfo(msgspec.Struct):
    """LineRangeInfo"""

    maxUnitValue: float
    minUnitValue: float
    interval: float
    marketUnit: str


class Description(msgspec.Struct):
    """Description"""

    persistenceEnabled: bool
    bspMarket: bool
    marketTime: str
    suspendTime: str
    bettingType: str
    turnInPlayEnabled: bool
    marketType: str
    regulator: str
    marketBaseRate: float
    discountAllowed: bool
    wallet: str
    rules: str
    rulesHasDate: bool
    priceLadderDescription: PriceLadderDescription
    lineRangeInfo: Optional[LineRangeInfo] = None


class EventType(msgspec.Struct):
    """EventType"""

    id: str
    name: str


class Competition(msgspec.Struct):
    """Competition"""

    id: str
    name: str


class Event(msgspec.Struct):
    """Event"""

    id: str
    name: str
    timezone: str
    openDate: str
    countryCode: Optional[str] = None


class Runner(msgspec.Struct):
    """
    https://docs.developer.betfair.com/display/1smk3cen4v3lu3yomq5qye0ni/Exchange+Stream+API
    """

    selectionId: int
    runnerName: str
    sortPriority: Optional[int] = None
    handicap: Optional[float] = None
    status: Optional[str] = None
    adjustmentFactor: Optional[float] = None
    metadata: Optional[dict] = None

    @property
    def runner_name(self):
        return self.runnerName

    @property
    def runner_id(self):
        if self.selectionId:
            return self.selectionId
        elif self.metadata.get("runnerId"):
            return int(self.metadata.get("runnerId"))
        return None


class MarketCatalog(msgspec.Struct):
    """MarketCatalog"""

    marketId: str
    marketName: str
    marketStartTime: str
    description: Description
    totalMatched: float
    runners: list[Runner]
    eventType: EventType
    event: Event
    competition: Optional[Competition] = None

    @property
    def competition_id(self) -> str:
        if not self.competition:
            return ""
        return self.competition.id

    @property
    def competition_name(self) -> str:
        if not self.competition:
            return ""
        return self.competition.name



from typing import List, Literal, Optional

import msgspec


class MatchedOrder(msgspec.Struct, frozen=True, array_like=True):
    price: float
    size: float


class UnmatchedOrder(msgspec.Struct, frozen=True):
    """
    https://docs.developer.betfair.com/display/1smk3cen4v3lu3yomq5qye0ni/Exchange+Stream+API
    """

    id: str
    p: float
    s: float
    side: Literal["B", "L"]
    status: Literal["E", "EC"]
    pt: str
    ot: str
    pd: int
    rac: str
    rc: str
    rfo: str
    rfs: str
    md: Optional[int] = None
    cd: Optional[int] = None
    ld: Optional[int] = None
    avp: Optional[float] = None
    sm: Optional[float] = None
    sr: Optional[float] = None
    sl: Optional[float] = None
    sc: Optional[float] = None
    sv: Optional[float] = None


class OrderChanges(msgspec.Struct):
    """
    https://docs.developer.betfair.com/display/1smk3cen4v3lu3yomq5qye0ni/Exchange+Stream+API
    """

    id: int
    fullImage: Optional[bool] = False
    hc: Optional[float] = None
    uo: Optional[List[UnmatchedOrder]] = []
    mb: Optional[List[MatchedOrder]] = []
    ml: Optional[List[MatchedOrder]] = []


class OrderAccountChange(msgspec.Struct):
    """
    https://docs.developer.betfair.com/display/1smk3cen4v3lu3yomq5qye0ni/Exchange+Stream+API
    """

    id: str
    fullImage: Optional[bool] = False
    orc: Optional[List[OrderChanges]] = []


class OCM(msgspec.Struct, tag_field="op", tag=str.lower):
    """
    https://docs.developer.betfair.com/display/1smk3cen4v3lu3yomq5qye0ni/Exchange+Stream+API
    """

    id: int
    clk: str
    pt: int
    oc: List[OrderAccountChange] = []



from typing import Literal

import msgspec


class APIBase(msgspec.Struct):
    def validate(self):
        return bool(msgspec.json.decode(msgspec.json.encode(self), type=type(self)))


class RequestBase(APIBase):
    jsonrpc: Literal["2.0"] = "2.0"
    id: int = 1



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['betfair_parser',
 'betfair_parser.spec',
 'betfair_parser.spec.api',
 'betfair_parser.spec.streaming']

package_data = \
{'': ['*']}

install_requires = \
['fsspec>=2022', 'msgspec>=0.12']

setup_kwargs = {
    'name': 'betfair-parser',
    'version': '0.1.13',
    'description': '',
    'long_description': 'None',
    'author': 'Bradley McElroy',
    'author_email': 'bradley.mcelroy@live.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from enum import Enum
from typing import List, Literal, Optional, Union

import msgspec

from betfair_parser.constants import EVENT_TYPE_TO_NAME


class RunnerStatus(Enum):
    ACTIVE = "ACTIVE"
    REMOVED = "REMOVED"
    WINNER = "WINNER"
    PLACED = "PLACED"
    LOSER = "LOSER"
    HIDDEN = "HIDDEN"


class MarketStatus(Enum):
    OPEN = "OPEN"
    SUSPENDED = "SUSPENDED"
    CLOSED = "CLOSED"


class RunnerValues(msgspec.Struct):
    """
    https://docs.developer.betfair.com/display/1smk3cen4v3lu3yomq5qye0ni/Exchange+Stream+API
    """

    tv: float  # Traded Volume
    ltp: float  # Last Traded Price
    spn: float  # Starting Price Near
    spf: float  # Starting Price Far


class Runner(msgspec.Struct):
    """
    https://docs.developer.betfair.com/display/1smk3cen4v3lu3yomq5qye0ni/Exchange+Stream+API
    """

    sortPriority: int
    id: Union[int, str]
    name: Optional[str] = None
    hc: Optional[Union[float, str]] = None
    status: Optional[RunnerStatus] = None
    adjustmentFactor: Optional[float] = None
    selectionId: Optional[str] = None
    bsp: Optional[Union[str, float]] = None

    @property
    def handicap(self) -> str:
        return str(self.hc or 0.0)

    @property
    def runner_id(self) -> int:
        return int(self.selectionId or self.id)


class MarketDefinition(msgspec.Struct, kw_only=True):
    """
    https://docs.developer.betfair.com/display/1smk3cen4v3lu3yomq5qye0ni/Exchange+Stream+API
    """

    bspMarket: bool
    turnInPlayEnabled: bool
    persistenceEnabled: bool
    marketBaseRate: Optional[float]
    marketId: Optional[str] = ""
    marketName: Optional[str] = ""
    eventId: str
    eventTypeId: str
    numberOfWinners: int
    bettingType: str
    marketType: str
    marketTime: str
    competitionId: Optional[str] = ""
    competitionName: Optional[str] = ""
    eventName: Optional[str] = ""
    suspendTime: str
    bspReconciled: bool
    complete: bool
    inPlay: bool
    crossMatching: bool
    runnersVoidable: bool
    numberOfActiveRunners: int
    betDelay: int
    status: MarketStatus
    runners: List[Runner]
    regulators: List[str]
    name: Optional[str] = None
    openDate: Optional[str] = None
    timezone: Optional[str] = None
    venue: Optional[str] = None
    version: Optional[int] = None
    countryCode: Optional[str] = None
    discountAllowed: Optional[bool] = None

    @property
    def event_type_name(self) -> str:
        return EVENT_TYPE_TO_NAME[self.eventTypeId]

    def to_dict(self):
        return {f: getattr(self, f) for f in self.__struct_fields__}


class AvailableToBack(msgspec.Struct, frozen=True, array_like=True):
    """AvailableToBack"""

    price: float
    volume: float


class AvailableToLay(msgspec.Struct, frozen=True, array_like=True):
    """AvailableToLay"""

    price: float
    volume: float


class BestAvailableToBack(msgspec.Struct, frozen=True, array_like=True):
    """BestAvailableToBack"""

    level: int
    price: float
    volume: float


class BestAvailableToLay(msgspec.Struct, frozen=True, array_like=True):
    """BestAvailableToLay"""

    level: int
    price: float
    volume: float


class BestDisplayAvailableToBack(msgspec.Struct, frozen=True, array_like=True):
    """BestDisplayAvailableToBack"""

    level: int
    price: float
    volume: float


class BestDisplayAvailableToLay(msgspec.Struct, frozen=True, array_like=True):
    """BestDisplayAvailableToLay"""

    level: int
    price: float
    volume: float


class Trade(msgspec.Struct, frozen=True, array_like=True):
    """Trade"""

    price: float
    volume: float


class StartingPriceBack(msgspec.Struct, frozen=True, array_like=True):
    """StartingPriceBack"""

    price: float
    volume: float


class StartingPriceLay(msgspec.Struct, frozen=True, array_like=True):
    """StartingPriceLay"""

    price: float
    volume: float


class RunnerChange(msgspec.Struct):
    """
    https://docs.developer.betfair.com/display/1smk3cen4v3lu3yomq5qye0ni/Exchange+Stream+API
    """

    id: Union[int, str]
    atb: Optional[List[AvailableToBack]] = []
    atl: Optional[List[AvailableToLay]] = []
    batb: Optional[List[BestAvailableToBack]] = []
    batl: Optional[List[BestAvailableToLay]] = []
    bdatb: Optional[List[BestDisplayAvailableToBack]] = []
    bdatl: Optional[List[BestDisplayAvailableToLay]] = []
    spb: Optional[List[StartingPriceBack]] = []  # Starting Price (Available To) Back
    spl: Optional[List[StartingPriceLay]] = []  # Starting Price (Available To) Lay
    spn: Optional[Union[float, str]] = None  # Starting Price Near
    spf: Optional[Union[int, float, str]] = None  # Starting Price Far
    trd: Optional[List[Trade]] = []
    ltp: Optional[float] = None
    tv: Optional[float] = None
    hc: Optional[float] = None


class MarketChange(msgspec.Struct):
    """
    https://docs.developer.betfair.com/display/1smk3cen4v3lu3yomq5qye0ni/Exchange+Stream+API
    """

    id: str
    marketDefinition: Optional[MarketDefinition] = None
    rc: List[RunnerChange] = []
    img: bool = False
    tv: Optional[float] = None
    con: Optional[bool] = None


class MCM(msgspec.Struct, tag_field="op", tag=str.lower):
    """
    https://docs.developer.betfair.com/display/1smk3cen4v3lu3yomq5qye0ni/Exchange+Stream+API
    """

    pt: int
    clk: Optional[str] = None
    id: Optional[int] = None
    initialClk: Optional[str] = None
    status: Optional[int] = None
    conflateMs: Optional[int] = None
    heartbeatMs: Optional[int] = None
    ct: Optional[Literal["HEARTBEAT", "SUB_IMAGE", "RESUB_DELTA"]] = None
    mc: List[MarketChange] = []

    @property
    def is_heartbeat(self):
        return self.ct == "HEARTBEAT"

    @property
    def stream_unreliable(self):
        return self.status == 503



from typing import Literal

from betfair_parser.spec.api.core import APIBase


class getAccountFunds(APIBase):
    method: Literal["AccountAPING/v1.0/getAccountFunds"] = "AccountAPING/v1.0/getAccountFunds"


class getAccountDetails(APIBase):
    method: Literal["AccountAPING/v1.0/getAccountDetails"] = "AccountAPING/v1.0/getAccountDetails"


class AccountDetails(APIBase):
    currencyCode: str
    firstName: str
    lastName: str
    localeCode: str
    region: str
    timezone: str
    discountRate: float
    pointsBalance: int
    countryCode: str


class AccountDetailsResponse(APIBase):
    result: AccountDetails


class AccountFunds(APIBase):
    availableToBetBalance: float
    exposure: float
    retainedCommission: float
    exposureLimit: float
    discountRate: float
    pointsBalance: int
    wallet: str


class AccountFundsResponse(APIBase):
    result: AccountFunds
